# -*- coding: utf-8 -*-

from odoo import models, fields, api, _, tools
import logging
from dateutil.relativedelta import relativedelta
from datetime import date
from odoo.exceptions import ValidationError
# Per importar funció is_valid_email que es troba en fitxer utils.py a la mateixa carpeta:
# from .utils import is_valid_email
# Per importar funció is_valid_email que es troba en fitxer utils.py a la carpeta de nivell superior:
from ..utils import is_valid_email

# Per importar funció is_valid_email que es troba en fitxer utils.py a la carpeta de 2 nivells per damunt:
# from ...utils import is_valid_email
# Per importar funció is_valid_email que es troba en fitxer utils.py en una subcarpeta xxx:
# from .xxx.utils import is_valid_email

_logger = logging.getLogger(__name__)


class SchoolTeacher(models.Model):
    _name = 'school.teacher'
    _description = 'Teacher Management'
    # _rec_name = 'full_name'    # No cal per haver incorporat name_get
    _order = 'last_name, first_name'
    _sql_constraints = [('tin_unique','unique(tin)',_('The Tax ID must be unique!'))]

    first_name = fields.Char('First Name', size=30, required=True)
    last_name = fields.Char('Last Name', size=40, required=True)
    birthdate = fields.Date('Birthdate', required=True)
    tin = fields.Char('Tax ID', size=14)
    gender = fields.Selection([('male', 'Male'), ('female', 'Female')], 'Gender')
    salary = fields.Integer('Salary')
    email = fields.Char('eMail', size=60, required=True)
    phone = fields.Char('Phone')
    course_ids = fields.One2many('school.course', 'manager_id', 'Managed Courses', readonly=True)
    subject_ids = fields.Many2many('school.subject', 'school_subject_teacher_rel', 'teacher_id',
                                   'subject_id', 'Subjects authorized', readonly=True)
    country_id = fields.Many2one('res.country', 'Citizenship', required=True,
                                 help='If has more than one citizenship, the main')
    active = fields.Boolean('Active', default=True)
    full_name = fields.Char('Full name', compute='_compute_full_name')
    age = fields.Integer('Age', compute='_compute_age')
    photo = fields.Binary('Photo')

    @api.depends('last_name', 'first_name')
    def _compute_full_name(self):
        # self dona accés al recordset de registres de la BD pels que s'ha de fer el càlcul que correspon
        for teacher in self:
            teacher.full_name = teacher.last_name + ', ' + teacher.first_name

    @api.depends('birthdate')
    def _compute_age(self):
        avui = date.today()
        for teacher in self:
            teacher.age = relativedelta(avui, teacher.birthdate).years

    @api.constrains('salary')
    def _check_salary(self):
        for teacher in self:
            if teacher.salary < 0:
                raise ValidationError(_('Salary must be positive'))

    @api.constrains('phone')
    def _check_phone(self):
        for teacher in self:
            # Possibilitat 1:
            # try:
            #     aux = int(teacher.phone)
            # except ValueError:
            # raise ValidationError( _('The phone can only have digits') )
            # Possibilitat 2:
            if teacher.phone != False and not teacher.phone.isdigit():
                raise ValidationError(_('The phone can only have digits'))
                # En aquesta 2a possiblitat, per preguntar per phone.isdigit(), cal estar
                # segur que phone tingui valor. Per això abans ens assegurem d'aquest fet

    @api.constrains('email')
    def _check_email(self):
        for teacher in self:
            if not is_valid_email(teacher.email):
                raise ValidationError(_('The eMail is not valid'))

    def name_get(self):
        result = []  # Llista buida que anirem emplenant per cada teacher
        for teacher in self:
            tupla = (teacher.id, teacher.last_name + ', ' + teacher.first_name)
            result.append(tupla)
        return result

    @api.onchange('tin')
    def _onchange_tin(self):
        if self.tin != False:
            # Només per què quan es dona d'alta o es deixa buit, doncs en no tenir valor
            # dona error en aplicar upper() sobre un null.
            self.tin = self.tin.upper()

class SchoolCourse(models.Model):
    _name = 'school.course'
    _description = 'Course Management'
    _order = 'name'

    name = fields.Char('Name', size=60, required=True)
    hours = fields.Integer('Hours', required=True)
    active = fields.Boolean('Active', default=True)
    manager_id = fields.Many2one('school.teacher', 'Course Manager', required=True,
        domain=[('country_id.code','=','ES')])
    manager_email = fields.Char('Manager eMail', related='manager_id.email')
    manager_phone = fields.Char('Manager phone', related='manager_id.phone')
    manager_citizenship = fields.Many2one('res.country', 'Man. Citizenship',
                                          related='manager_id.country_id')
    course_subject_ids = fields.One2many('school.course.subject', 'course_id', 'Subjects',
                                         readonly=True)
    thematic_id = fields.Many2one('school.thematic', 'Thematic', required=True)
    summary = fields.Text('Summary')
    edition_ids = fields.One2many('school.course.edition', 'course_id', 'Editions', readonly=True)

    @api.constrains('hours')
    def _check_hours(self):
        for course in self:
            if course.hours <= 0:
                raise ValidationError(_('The hours of the course must be strictly positive'))

    def _auto_init(self):
        res = super(SchoolCourse, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_course_unique_lower_name',
                                  self._table, ['lower(name)'])
        return res

class SchoolSubject(models.Model):
    _name = 'school.subject'
    _description = 'Subject Management'
    _order = 'name'

    name = fields.Char('Name', size=60, required=True)
    hours = fields.Integer('Hours', required=True)
    active = fields.Boolean('Active', default=True)
    course_subject_ids = fields.One2many('school.course.subject', 'subject_id', 'Courses',
                                         readonly=True)
    teacher_ids = fields.Many2many('school.teacher', 'school_subject_teacher_rel', 'subject_id', 'teacher_id',
                                   'Authorized Teachers', readonly=True)

    @api.constrains('hours')
    def _check_hours(self):
        for course in self:
            if course.hours <= 0:
                raise ValidationError(_('The hours of the subject must be strictly positive'))

    def _auto_init(self):
        res = super(SchoolSubject, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_subject_unique_lower_name',
                                  self._table, ['lower(name)'])
        return res

class SchoolCourseSubject(models.Model):
    _name = 'school.course.subject'
    _description = 'Course-Subject Management'
    _order = 'number'
    _sql_constraints=[('course_subject_unique','unique(course_id, subject_id)',_('The subject in a course must be unique!')),
                      ('course_number_unique','unique(course_id, number)',_('The number in a course must be unique!'))
                      ]

    course_id = fields.Many2one('school.course', 'Course', required=True, ondelete='cascade')
    subject_id = fields.Many2one('school.subject', 'Subject', required=True, ondelete='restrict')
    number = fields.Integer('Number', required=True)

    @api.constrains('number')
    def _check_number(self):
        for record in self:
            if record.number <= 0:
                raise ValidationError(_('Number must be greater than zero'))


class SchoolThematic(models.Model):
    _name = 'school.thematic'
    _description = 'Thematic Management'
    _order = 'name'

    name = fields.Char('Name', size=30, required=True)
    course_ids = fields.One2many('school.course', 'thematic_id', 'Courses', readonly=True)
    parent_id = fields.Many2one('school.thematic', 'Parent Thematic')
    child_ids = fields.One2many('school.thematic', 'parent_id', 'Child Thematics', readonly=True)

    def _auto_init(self):
        res = super(SchoolThematic, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_thematic_unique_lower_name',
                                  self._table, ['lower(name)'])
        return res


class SchoolCourseEdition(models.Model):
    _name = 'school.course.edition'
    _description = 'Course-Edition Management'
    _order = 'name'

    name = fields.Char('Edition name', required=True)
    date_start = fields.Date('Init date', required=True)
    date_end = fields.Date('End date')
    course_id = fields.Many2one('school.course', 'Course', required=True, ondelete='cascade')

    @api.constrains('date_start', 'date_end')
    def _check_dates(self):
        for ce in self:
            if ce.date_end != False and ce.date_end < ce.date_start:
                raise ValidationError(_("Date end must be later or equal date start"))

    def _auto_init(self):
        res = super(SchoolCourseEdition, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_course_edition_unique_lower_name',
                                  self._table, ['course_id, lower(name)'])
