# -*- coding: utf-8 -*-

from odoo import models, fields, api, _, tools
import logging
from dateutil.relativedelta import relativedelta
from datetime import date, datetime
from odoo.exceptions import ValidationError, UserError
# Per importar funció is_valid_email que es troba en fitxer utils.py a la mateixa carpeta:
# from .utils import is_valid_email
# Per importar funció is_valid_email que es troba en fitxer utils.py a la carpeta de nivell superior:
from ..utils import is_valid_email

# Per importar funció is_valid_email que es troba en fitxer utils.py a la carpeta de 2 nivells per damunt:
# from ...utils import is_valid_email
# Per importar funció is_valid_email que es troba en fitxer utils.py en una subcarpeta xxx:
# from .xxx.utils import is_valid_email

_logger = logging.getLogger(__name__)


class SchoolTeacher(models.Model):
    _name = 'school.teacher'
    _description = 'Teacher Management'
    # _rec_name = 'full_name'    # No cal per haver incorporat name_get
    _order = 'last_name, first_name'
    _sql_constraints = [('tin_unique','unique(tin)',_('The Tax ID must be unique!'))]

    first_name = fields.Char('First Name', size=30, required=True)
    last_name = fields.Char('Last Name', size=40, required=True)
    birthdate = fields.Date('Birthdate', required=True)
    # Camps que afegim per la vista calendar (aniversari dins l'any actual)
    birthday = fields.Date('Birthday', compute='_compute_birthday')
    birthday_age = fields.Integer('Age', compute='_compute_birthday_age')
    tin = fields.Char('Tax ID', size=14)
    gender = fields.Selection([('male', 'Male'), ('female', 'Female')], 'Gender')
    salary = fields.Integer('Salary')
    email = fields.Char('eMail', size=60, required=True)
    phone = fields.Char('Phone')
    course_ids = fields.One2many('school.course', 'manager_id', 'Managed Courses', readonly=True)
    subject_ids = fields.Many2many('school.subject', 'school_subject_teacher_rel', 'teacher_id',
                                   'subject_id', 'Subjects authorized', readonly=True)
    country_id = fields.Many2one('res.country', 'Citizenship', required=True,
                                 help='If has more than one citizenship, the main')
    active = fields.Boolean('Active', default=True)
    full_name = fields.Char('Full name', compute='_compute_full_name')
    age = fields.Integer('Age', compute='_compute_age')
    photo = fields.Binary('Photo')
    n_managed = fields.Integer('N. Courses', compute='_compute_n_managed')
    n_subject = fields.Integer('N. Subjects', compute='_compute_n_subject')
    n_teaching = fields.Integer('N. Teaching', compute='_compute_n_teaching')

    @api.depends('last_name', 'first_name')
    def _compute_full_name(self):
        # self dona accés al recordset de registres de la BD pels que s'ha de fer el càlcul que correspon
        for teacher in self:
            teacher.full_name = teacher.last_name + ', ' + teacher.first_name

    @api.depends('birthdate')
    def _compute_birthday(self):
        # self dona accés al recordset de registres de la BD pels que s'ha de fer el càlcul que correspon
        for teacher in self:
            aa = date.today().year     # any actual
            if teacher.birthdate.day==29 and teacher.birthdate.month==2:
                # Nascut el 29 de febrer
                if aa % 4 == 0 and aa % 100 !=0 or aa % 400 == 0:
                    # L'any actual és de traspàs => l'aniversari cau en 29 de febrer
                    teacher.birthday = teacher.birthdate.replace(year=aa)
                else:
                    # L'aniversari cau en 1 de març
                    teacher.birthday = teacher.birthdate.replace(year=aa, month=3, day=1)
            else:
                # Nascut en qualsevol dia que no és 29 de febrer
                teacher.birthday = teacher.birthdate.replace(year=aa)

    @api.depends('birthdate')
    def _compute_age(self):
        avui = date.today()
        for teacher in self:
            teacher.age = relativedelta(avui, teacher.birthdate).years

    @api.depends('birthdate')
    def _compute_birthday_age(self):
        for teacher in self:
            teacher.birthday_age = relativedelta(teacher.birthday, teacher.birthdate).years

    @api.constrains('salary')
    def _check_salary(self):
        for teacher in self:
            if teacher.salary < 0:
                raise ValidationError(_('Salary must be positive'))

    @api.constrains('phone')
    def _check_phone(self):
        for teacher in self:
            # Possibilitat 1:
            # try:
            #     aux = int(teacher.phone)
            # except ValueError:
            # raise ValidationError( _('The phone can only have digits') )
            # Possibilitat 2:
            if teacher.phone != False and not teacher.phone.isdigit():
                raise ValidationError(_('The phone can only have digits'))
                # En aquesta 2a possiblitat, per preguntar per phone.isdigit(), cal estar
                # segur que phone tingui valor. Per això abans ens assegurem d'aquest fet

    @api.constrains('email')
    def _check_email(self):
        for teacher in self:
            if not is_valid_email(teacher.email):
                raise ValidationError(_('The eMail is not valid'))

    def name_get(self):
        result = []  # Llista buida que anirem emplenant per cada teacher
        for teacher in self:
            tupla = (teacher.id, teacher.last_name + ', ' + teacher.first_name)
            result.append(tupla)
        return result

    @api.onchange('tin')
    def _onchange_tin(self):
        if self.tin != False:
            # Només per què quan es dona d'alta o es deixa buit, doncs en no tenir valor
            # dona error en aplicar upper() sobre un null.
            self.tin = self.tin.upper()

    @api.depends('course_ids')
    def _compute_n_managed(self):
        for teacher in self:
            teacher.n_managed = len(teacher.course_ids)

    @api.depends('subject_ids')
    def _compute_n_subject(self):
        for teacher in self:
            teacher.n_subject = len(teacher.subject_ids)

    def _compute_n_teaching(self):
        for teacher in self:
            teacher.n_teaching = self.env['school.teaching'].search_count([('teacher_id','=',teacher.id)])

    def unlink(self):
        # self conté els registres a eliminar
        for teacher in self:
            # Comprovar si el teacher és responsable de cursos
            # No ens serveix consultar len(teacher.course_ids) ni n_managed, per què no comptabilitzen els
            # cursos arxivats. ALERTA: Sí que es comptabilitzen si la vista te "active_test" a True (Odoo 16)
            # però NO es veuen en la vista "tree"... Solució: Fer el comptatge amb un search_count:
            q = self.env['school.course'].search_count([('manager_id','=',teacher.id),'|',('active','=',True),('active','=',False)])
            if q>0:
                raise UserError(_('You cannot delete any teacher who he is manager of any course (active or inactive)'))
        # Arribats aquí, ja hem comprovat que els profes a eliminar no són responsables de cap curs
        # Hem d'eliminar les possibles docències que tinguin:
        # Cerquem els "id" dels teachers dels que volem eliminar les docències
        id_teachers = []
        for teacher in self:
            id_teachers.append(teacher.id)
        # Cerquem les docències afectades:
        teachings = self.env['school.teaching'].search([('teacher_id','in',id_teachers)])
        # Eliminem les docències afectades:
        teachings.unlink()
        # Eliminem el teacher
        q = super(SchoolTeacher,self).unlink()
        # El mètode unlink ha de finalitzar indicant la qtat d'objectes eliminats
        return q

    @api.model_create_multi
    def create (self, values):
        r = super(SchoolTeacher, self).create(values)
        # values pot ser una llista de diccionaris (en cas de creació simultània de varis registres).
        # I si fos així, r seria la llista de teachers creats. Per tant, cal procedir per cada possible registre creat
        # Recordar que en un mètode create, values encara no conté "id" de l'objecte que es crearà...
        # momentActual = datetime.now()   # Observem que Odoo dona l'hora en UTC i no és el què es vol!!!
        momentActual = fields.Datetime.context_timestamp(self,datetime.now())
        for teacher in r: # Per cada teacher inserit
            nou_history = {}
            nou_history['teacher_id']=teacher.id
            nou_history['user_id']=self.env.uid
            nou_history['date']=momentActual.strftime('%Y-%m-%d')
            nou_history['time_s']=momentActual.strftime('%H:%M')
            nou_history['time_f']=int(momentActual.strftime('%H')) + float(momentActual.strftime('%M')) / 60
            nou_history['salary']=teacher.salary
            self.env['school.teacher.salary.history'].create(nou_history)
        return r

    def write(self,values):
        # values és un diccionari que conté els valors dels camps a modificar
        # Només conté els camps que es modifiquen, NO tots
        # self apunta a tots els registres que es modifiquen
        r = super(SchoolTeacher, self).write(values)
        if r == True and 'salary' in values.keys():
            momentActual = fields.Datetime.context_timestamp(self, datetime.now())
            # datetime.now() seria la data-hora que Odoo sempre dona en el fus UTC
            # momentActual és la data-hora que té l'usuari a les seves preferències
            for teacher in self:  # Per cada teacher modificat
                nou_history = {}
                nou_history['teacher_id'] = teacher.id
                nou_history['user_id'] = self.env.uid
                nou_history['date'] = momentActual.strftime('%Y-%m-%d')
                nou_history['time_s'] = momentActual.strftime('%H:%M')
                nou_history['time_f'] = int(momentActual.strftime('%H')) + float(momentActual.strftime('%M')) / 60
                nou_history['salary'] = teacher.salary
                self.env['school.teacher.salary.history'].create(nou_history)
        return r


class SchoolCourse(models.Model):
    _name = 'school.course'
    _description = 'Course Management'
    _order = 'name'

    name = fields.Char('Name', size=60, required=True)
    hours = fields.Integer('Hours', required=True)
    active = fields.Boolean('Active', default=True)
    manager_id = fields.Many2one('school.teacher', 'Course Manager', required=True,
        domain=[('country_id.code','=','ES')])
    manager_email = fields.Char('Manager eMail', related='manager_id.email')
    manager_phone = fields.Char('Manager phone', related='manager_id.phone')
    manager_citizenship = fields.Many2one('res.country', 'Man. Citizenship',
                                          related='manager_id.country_id')
    course_subject_ids = fields.One2many('school.course.subject', 'course_id', 'Subjects',
                                         readonly=True)
    thematic_id = fields.Many2one('school.thematic', 'Thematic', required=True)
    summary = fields.Text('Summary')
    edition_ids = fields.One2many('school.course.edition', 'course_id', 'Editions', readonly=True)

    @api.constrains('hours')
    def _check_hours(self):
        for course in self:
            if course.hours <= 0:
                raise ValidationError(_('The hours of the course must be strictly positive'))

    def _auto_init(self):
        res = super(SchoolCourse, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_course_unique_lower_name',
                                  self._table, ['lower(name)'])
        return res

    def write(self,values):
        if 'manager_id' in values.keys() and values['manager_id'] == False:
            raise UserError(_('Process aborted because a course cannot be left without a responsible teacher'))
        r = super(SchoolCourse,self).write(values)
        return r

class SchoolSubject(models.Model):
    _name = 'school.subject'
    _description = 'Subject Management'
    _order = 'name'

    name = fields.Char('Name', size=60, required=True)
    hours = fields.Integer('Hours', required=True)
    active = fields.Boolean('Active', default=True)
    course_subject_ids = fields.One2many('school.course.subject', 'subject_id', 'Courses',
                                         readonly=True)
    teacher_ids = fields.Many2many('school.teacher', 'school_subject_teacher_rel', 'subject_id', 'teacher_id',
                                   'Authorized Teachers', readonly=True)

    @api.constrains('hours')
    def _check_hours(self):
        for course in self:
            if course.hours <= 0:
                raise ValidationError(_('The hours of the subject must be strictly positive'))

    def _auto_init(self):
        res = super(SchoolSubject, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_subject_unique_lower_name',
                                  self._table, ['lower(name)'])
        return res

class SchoolCourseSubject(models.Model):
    _name = 'school.course.subject'
    _description = 'Course-Subject Management'

    _order = 'number'
    _sql_constraints=[('course_subject_unique','unique(course_id, subject_id)',_('The subject in a course must be unique!')),
                      ('course_number_unique','unique(course_id, number)',_('The number in a course must be unique!'))
                      ]

    course_id = fields.Many2one('school.course', 'Course', required=True, ondelete='cascade')
    subject_id = fields.Many2one('school.subject', 'Subject', required=True, ondelete='restrict')
    number = fields.Integer('Number', required=True)
    subject_hours = fields.Integer('Hours', related='subject_id.hours')

    @api.constrains('number')
    def _check_number(self):
        for record in self:
            if record.number <= 0:
                raise ValidationError(_('Number must be greater than zero'))

    def name_get(self):
        result = []  # Llista buida que anirem emplenant per cada CourseSubject
        for cs in self:
            tupla = (cs.id, cs.course_id.name + " - " + str(cs.number) + " - " + cs.subject_id.name)
            result.append(tupla)
        return result

class SchoolThematic(models.Model):
    _name = 'school.thematic'
    _description = 'Thematic Management'
    _order = 'name'

    name = fields.Char('Name', size=30, required=True, translate=True)
    course_ids = fields.One2many('school.course', 'thematic_id', 'Courses', readonly=True)
    parent_id = fields.Many2one('school.thematic', 'Parent Thematic')
    child_ids = fields.One2many('school.thematic', 'parent_id', 'Child Thematics', readonly=True)

    def _auto_init(self):
        res = super(SchoolThematic, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_thematic_unique_lower_name',
                                  self._table, ['lower(name)'])
        return res


class SchoolCourseEdition(models.Model):
    _name = 'school.course.edition'
    _description = 'Course-Edition Management'
    _order = 'date_start'

    _sql_constraints = {
        ('unique_course_date_start', 'unique(course_id, date_start)',
         _('It''s not possible 2 course editions with same date start'))
    }

    name = fields.Char('Edition name', required=True)
    date_start = fields.Date('Init date', required=True)
    date_end = fields.Date('End date')
    course_id = fields.Many2one('school.course', 'Course', required=True, ondelete='cascade')

    @api.constrains('date_start', 'date_end')
    def _check_dates(self):
        for ce in self:
            if ce.date_end != False and ce.date_end < ce.date_start:
                raise ValidationError(_("Date end must be later or equal date start"))

    def name_get(self):
        result = []  # Llista buida que anirem emplenant per cada CourseEdition
        for ce in self:
            tupla = (ce.id, ce.course_id.name + ' - ' + ce.name)
            result.append(tupla)
        return result

    def _auto_init(self):
        res = super(SchoolCourseEdition, self)._auto_init()
        tools.create_unique_index(self._cr, 'school_course_edition_unique_lower_name',
                                  self._table, ['course_id, lower(name)'])

class SchoolTeaching(models.Model):
    _name = 'school.teaching'
    _description = 'Teaching Management (Teacher-Edition-Subject)'

    _sql_constraints = {
        ('unique_edition_teacher_subject','unique(edition_id, teacher_id, subject_id)',
         _('It is not possible to repeat the teaching of a teacher in a subject in an edition'))
    }

    teacher_id = fields.Many2one('school.teacher', 'Teacher', required=True)
    edition_id = fields.Many2one('school.course.edition','Course Edition', required=True)
    subject_id = fields.Many2one('school.course.subject','Subject', required=True)
    edition_course_id = fields.Many2one('school.course', related="edition_id.course_id")
    subject_teacher_ids = fields.Many2many('school.teacher', related='subject_id.subject_id.teacher_ids')

class SchoolTeacherSalaryHistory(models.Model):
    _name = 'school.teacher.salary.history'
    _description = 'History of Teacher Salaries'

    # order = 'date desc, time_f desc'
    order = 'id desc'

    teacher_id = fields.Many2one('school.teacher','Teacher', required=True, ondelete='cascade')
    user_id = fields.Many2one('res.users','User',required=True, ondelete='restrict')
    # No caldria 'restrict'
    date = fields.Date('Date',required=True)
    time_f = fields.Float('Time-F',digits=(5,2), required=True)
    time_s = fields.Char('Time-S', size=5, required=True)
    salary = fields.Integer('Salary', required=True)

